// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

part of 'password_reset_cubit.dart';

/// {@template password_reset_state}
/// The state of the [PasswordResetCubit].
/// {@endtemplate}
class PasswordResetState extends FormDataState {
  /// {@macro password_reset_state}
  const PasswordResetState({
    required super.form,
    super.status = FormStatus.pure,
    super.errorMessage,
  });

  /// The email validator of the form.
  Email get email => form.validatorOf(AuthFormField.email);

  PasswordResetState copyWith({
    WyattForm? form,
    FormStatus? status,
    String? errorMessage,
  }) =>
      PasswordResetState(
        form: form ?? this.form,
        status: status ?? this.status,
        errorMessage: errorMessage ?? this.errorMessage,
      );

  @override
  List<Object> get props => [email, status];

  @override
  String toString() => 'PasswordResetState(status: ${status.name} '
      '${(errorMessage != null) ? " [$errorMessage]" : ""}, $form)';
}
