// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:wyatt_ui_components/wyatt_ui_components.dart';
import 'package:wyatt_ui_kit/src/components/gradients/gradient_text_style.dart';

class RichTextBuilderThemeExtensionImpl extends RichTextBuilderThemeExtension {
  const RichTextBuilderThemeExtensionImpl({
    super.defaultStyle,
    super.styles,
  });

  factory RichTextBuilderThemeExtensionImpl.dark({ThemeData? theme}) {
    theme ??= ThemeData.dark();

    return RichTextBuilderThemeExtensionImpl(
      defaultStyle: theme.textTheme.bodyMedium?.copyWith(
        height: 1.8,
      ),
      styles: stylesFor(theme),
    );
  }

  factory RichTextBuilderThemeExtensionImpl.light({ThemeData? theme}) {
    theme ??= ThemeData.light();

    return RichTextBuilderThemeExtensionImpl(
      defaultStyle: theme.textTheme.bodyMedium?.copyWith(
        height: 1.8,
      ),
      styles: stylesFor(theme),
    );
  }

  static Map<String, TextStyle> stylesFor(ThemeData theme) => {
        'gradient-blue': GradientTextStyle.from(
          theme.textTheme.bodyMedium?.copyWith(
            fontWeight: FontWeight.w600,
            height: 1.8,
          ),
          const MultiColor([
            Color(0xFF3C97FB),
            Color(0xFF436EF4),
          ]),
        ),
        'gradient-red': GradientTextStyle.from(
          theme.textTheme.bodyMedium?.copyWith(
            fontWeight: FontWeight.w600,
            height: 1.8,
          ),
          const MultiColor([
            Color(0xFFF44464),
            Color(0xFFF44464),
          ]),
        ),
        'gradient-green': GradientTextStyle.from(
          theme.textTheme.bodyMedium?.copyWith(
            fontWeight: FontWeight.w600,
            height: 1.8,
          ),
          const MultiColor([
            Color(0xFF00D16C),
            Color(0xFF00D16C),
          ]),
        ),
        'blue': theme.textTheme.bodyMedium?.copyWith(
              fontWeight: FontWeight.w600,
              color: const Color(0xFF3C97FB),
              height: 1.8,
            ) ??
            const TextStyle(
              color: Color(0xFF3C97FB),
            ),
        'red': theme.textTheme.bodyMedium?.copyWith(
              fontWeight: FontWeight.w600,
              color: const Color(0xFFF44464),
              height: 1.8,
            ) ??
            const TextStyle(
              color: Color(0xFFF44464),
            ),
        'green': theme.textTheme.bodyMedium?.copyWith(
              fontWeight: FontWeight.w600,
              color: const Color(0xFF00D16C),
              height: 1.8,
            ) ??
            const TextStyle(
              color: Color(0xFF00D16C),
            ),
      };
}
